////////////////////////////////////////////////////////
// DLL and unix - Dynamic link
//
// Sylvain Marechal
////////////////////////////////////////////////////////
#include <stdio.h>
#include <pthread.h>
#include <stdlib.h>
#include <dlfcn.h>

typedef void (*pf_shared_entry_point)( int  );


double main_f1()
{
    int i;
    double d = 1.75;
    for( i = 0; i < 50000; i ++ )
    {
        d = ((d + 1) + d)/2;
    }

    printf( "main - d = %g (%g)\n", d, 123.456 );
    return d;
}

double main_f2()
{
    int i;
    double d = 1.75;
    for( i = 0; i < 25000; i ++ )
    {
        d = ((d + 1) + d)/2;
    }

    printf( "main - d = %g (%g)\n", d, 123.456 );
    return d;
}

double main_f3()
{
    int i;
    double d = 1.75;
    for( i = 0; i < 100000; i ++ )
    {
        d = ((d + 1) + d)/2;
    }

    printf( "main - d = %g (%g)\n", d, 123.456 );
    return d;
}

void main_f123()
{
    int i;
    for( i = 0; i < 10; i ++ ) main_f1();
    for( i = 0; i < 10; i ++ ) main_f2();
//    for( i = 0; i < 10; i ++ ) main_f1();
    for( i = 0; i < 10; i ++ ) main_f3();
}


void * thread_func( void * param)
{
    int *pend = (void *)param;
    printf( "in the thread function\n" );
    main_f123();
    *pend = 1;
    return NULL;
}

int make_thread()
{
    pthread_t  th; 
    int end = 0;

    pthread_create(&th, NULL, (void *)&thread_func, (void *)&end ); 
    pthread_detach(th); 
    
    while( end == 0 ) sleep(1); 
}

int main(void) 
{
    const char *error;
    void *module;
    pf_shared_entry_point pf;

#if 1
    /* test profiling with a thread */
    make_thread();
#else
    /* test profiling without thread */
    main_f123();
#endif
   
#if 1
    /* Load dynamically loaded library */
    module = dlopen("./libshared.so", RTLD_LAZY);
    if (!module) 
    {
	fprintf(stderr, "Couldn't open the .so: %s\n",
	    dlerror());
	exit(1);
    }
    
    /* Get symbol */
    dlerror();
    pf = (pf_shared_entry_point)dlsym(module, "shared_entry_point");
    if ((error = dlerror())) 
    {
	fprintf(stderr, "Couldn't find hello: %s\n", error);
	exit(1);
    }
    
    /* Now call the function in the DL library */
    (*pf)(8888);
    
    /* All done, close things cleanly */
    dlclose(module);
#else
    /* in case shared is linked statically with main */
    shared_enty_point(1234);
#endif
    return 0;
}
